@@ -22,7 +22,6 @@
 
 #include <algorithm>
 #include <functional>
-#include <string>
 
 #include <giomm.h>
 #include <glibmm/i18n.h>
@@ -30,33 +29,10 @@
 
 #include "export-queue-operations.h"
 #include "main-window.h"
-#include "progress-observer.h"
 
 namespace Solang
 {
 
-static const std::string artistsFile = PACKAGE_DOC_DIR"/ARTISTS";
-static const std::string authorsFile = PACKAGE_DOC_DIR"/AUTHORS";
-static const std::string copyingFile = PACKAGE_DOC_DIR"/COPYING";
-
-static const std::string iconFile
-    = PACKAGE_DATA_DIR"/icons/hicolor/22x22/apps/"
-          PACKAGE_TARNAME".png";
-
-static const std::string layoutFile
-    = PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/"
-          PACKAGE_TARNAME"-layout.xml";
-
-static const std::string logoFile
-    = PACKAGE_DATA_DIR"/icons/hicolor/96x96/apps/"
-          PACKAGE_TARNAME".png";
-
-static const std::string uiFile
-    = PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/ui/"PACKAGE_TARNAME".ui";
-
-static const Glib::ustring websiteURL
-    = "http://projects.gnome.org/solang";
-
 class Docker :
     public std::unary_function<DockObjectPtr const &, void>
 {
@@ -117,6 +93,7 @@ void
 Docker::operator()(DockObjectPtr const & requester) throw()
 {
     gdl_dock_object_dock(reference_, requester, placement_, NULL);
+    gtk_widget_show_all(GTK_WIDGET(requester));
 }
 
 class DockHider :
@@ -192,9 +169,31 @@ DockHider::operator()(DockObjectPtr const & dock_object) throw()
         return;
     }
 
+    gtk_widget_hide(GTK_WIDGET(dock_object));
     gdl_dock_item_hide_item(GDL_DOCK_ITEM(dock_object));
 }
 
+const std::string MainWindow::artistsFile_
+    = PACKAGE_DOC_DIR G_DIR_SEPARATOR_S "ARTISTS";
+
+const std::string MainWindow::authorsFile_
+    = PACKAGE_DOC_DIR G_DIR_SEPARATOR_S "AUTHORS";
+
+const std::string MainWindow::copyingFile_
+    = PACKAGE_DOC_DIR G_DIR_SEPARATOR_S "COPYING";
+
+const std::string MainWindow::layoutFile_
+                      = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                        PACKAGE_TARNAME"-layout.xml";
+
+const std::string MainWindow::uiFile_
+                      = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                        "ui" G_DIR_SEPARATOR_S
+                        PACKAGE_TARNAME".ui";
+
+const Glib::ustring MainWindow::websiteURL_
+                        = "http://projects.gnome.org/solang";
+
 MainWindow::MainWindow() throw() :
     Gtk::Window(Gtk::WINDOW_TOPLEVEL),
     application_(),
@@ -203,13 +202,11 @@ MainWindow::MainWindow() throw() :
                                             __FILE__,
                                             __LINE__))),
     uiManager_(Gtk::UIManager::create()),
-    uiID_(uiManager_->add_ui_from_file(uiFile)),
+    uiID_(uiManager_->add_ui_from_file(uiFile_)),
     vBox_(false, 0),
     spinnerToolItem_(),
     hBox_(false, 6),
     statusBar_(),
-    progress_(),
-    observer_(),
     dock_(gdl_dock_new()),
     dockBar_(gdl_dock_bar_new(GDL_DOCK(dock_))),
     layout_(gdl_dock_layout_new(GDL_DOCK(dock_))),
@@ -219,28 +216,7 @@ MainWindow::MainWindow() throw() :
     dockObjectsLeftBottom_(),
     dockObjectsCenter_()
 {
-    PixbufPtr icon;
-
-    try
-    {
-        icon = Gdk::Pixbuf::create_from_file(iconFile);
-    }
-    catch (const Glib::FileError & e)
-    {
-        g_warning("%s", e.what().c_str());
-        return;
-    }
-    catch (const Gdk::PixbufError & e)
-    {
-        g_warning("%s", e.what().c_str());
-        return;
-    }
-
-    if (0 != icon)
-    {
-        set_icon(icon);
-    }
-
+    set_icon_name(PACKAGE_TARNAME);
     set_title("Solang");
     set_default_size(800, 600);
     add(vBox_);
@@ -312,6 +288,12 @@ MainWindow::MainWindow() throw() :
         Gtk::Action::create(
             "ActionHelpMenu", _("_Help")));
 
+    actionGroup_->add(
+        Gtk::Action::create(
+            "ActionHelpContents", Gtk::Stock::HELP,
+            _("_Contents")),
+        Gtk::AccelKey("F1"),
+        sigc::mem_fun(*this, &MainWindow::on_action_help_contents));
 
     actionGroup_->add(
         Gtk::Action::create(
@@ -346,7 +328,6 @@ MainWindow::MainWindow() throw() :
 
     statusBar_.set_has_resize_grip(true);
     vBox_.pack_start(statusBar_, Gtk::PACK_SHRINK, 0);
-    statusBar_.pack_start(progress_, Gtk::PACK_SHRINK, 0);
 
     show_all_children();
 }
@@ -368,6 +349,8 @@ MainWindow::init(Application & application) throw()
         gdl_dock_add_item(GDL_DOCK(dock_),
                           GDL_DOCK_ITEM(dockObjectsCenter_.front()),
                           GDL_DOCK_RIGHT);
+        gtk_widget_show_all(GTK_WIDGET(dockObjectsCenter_.front()));
+
         if (1 < dockObjectsCenter_.size())
         {
             std::for_each(++dockObjectsCenter_.begin(),
@@ -383,6 +366,8 @@ MainWindow::init(Application & application) throw()
         gdl_dock_add_item(GDL_DOCK(dock_),
                           GDL_DOCK_ITEM(dockObjectsLeftTop_.front()),
                           GDL_DOCK_LEFT);
+        gtk_widget_show_all(GTK_WIDGET(dockObjectsLeftTop_.front()));
+
         if (1 < dockObjectsLeftTop_.size())
         {
             std::for_each(++dockObjectsLeftTop_.begin(),
@@ -397,6 +382,8 @@ MainWindow::init(Application & application) throw()
         gdl_dock_add_item(GDL_DOCK(dock_),
             GDL_DOCK_ITEM(dockObjectsLeftBottom_.front()),
             GDL_DOCK_BOTTOM);
+        gtk_widget_show_all(GTK_WIDGET(dockObjectsLeftBottom_.front()));
+
         if (1 < dockObjectsLeftBottom_.size())
         {
             std::for_each(++dockObjectsLeftBottom_.begin(),
@@ -406,23 +393,45 @@ MainWindow::init(Application & application) throw()
         }
     }
 
-    const bool result = gdl_dock_layout_load_from_file(
-                            GDL_DOCK_LAYOUT(layout_),
-                            layoutFile.c_str());
+    load_layout();
 
-    if (true == result)
-    {
-        gdl_dock_layout_load_layout(GDL_DOCK_LAYOUT(layout_), 0);
-    }
-    else
-    {
-        g_warning("%s: File not found", layoutFile.c_str());
-    }
+    present_dock_object(dockObjectsCenter_.front());
 }
 
 void
 MainWindow::final(Application & application) throw()
 {
+    // NB: + Hide all renderers except the browser before saving the
+    //       layout. Otherwise, all hell breaks loose when loading
+    //       the layout because GDL tries to place the renderers in a
+    //       Gtk::Notebook, which leads to spurious switch-page
+    //       signals being emitted.
+    //     + Do not save the layout if something other than a renderer
+    //       has been docked in the centre. In other words, when all
+    //       the other renderers have been hidden, the browser should
+    //       not be placed inside a Gtk::Notebook. The reason for this
+    //       is the same as above.
+
+    if (1 < dockObjectsCenter_.size())
+    {
+        const std::vector<DockObjectPtr>::iterator second
+            = dockObjectsCenter_.begin() + 1;
+
+        std::for_each(second, dockObjectsCenter_.end(), DockHider());
+        dockObjectsCenter_.erase(second, dockObjectsCenter_.end());
+
+        Gtk::Container * const dock_object
+            = Glib::wrap(GTK_CONTAINER(dockObjectsCenter_.front()),
+                         false);
+
+        if (0 != dynamic_cast<Gtk::Notebook *>(
+                     dock_object->get_parent()))
+        {
+            return;
+        }
+    }
+
+    save_layout();
 }
 
 void
@@ -450,6 +459,7 @@ MainWindow::dock_object_center(DockObjectPtr dock_object) throw()
 {
     gdl_dock_object_dock(dockObjectsCenter_.front(),
                          dock_object, GDL_DOCK_CENTER, NULL);
+    dockObjectsCenter_.push_back(dock_object);
 }
 
 void
@@ -461,7 +471,18 @@ MainWindow::undock_object_center(DockObjectPtr dock_object) throw()
         return;
     }
 
+    const std::vector<DockObjectPtr>::iterator iter
+        = std::find(dockObjectsCenter_.begin(),
+                    dockObjectsCenter_.end(),
+                    dock_object);
+
+    if (dockObjectsCenter_.end() == iter)
+    {
+        return;
+    }
+
     gdl_dock_item_hide_item(GDL_DOCK_ITEM(dock_object));
+    dockObjectsCenter_.erase(iter);
 }
 
 void
@@ -516,6 +537,29 @@ MainWindow::get_user_layout_file() throw()
                + Glib::get_prgname() + "-layout.xml";
 }
 
+void
+MainWindow::load_layout() throw()
+{
+    if (false == gdl_dock_layout_load_from_file(
+                    GDL_DOCK_LAYOUT(layout_),
+                    get_user_layout_file().c_str()))
+    {
+        if (false == gdl_dock_layout_load_from_file(
+                         GDL_DOCK_LAYOUT(layout_),
+                         layoutFile_.c_str()))
+        {
+            g_warning("gdl_dock_layout_load_from_file: Failed");
+            return;
+        }
+    }
+
+    if (false == gdl_dock_layout_load_layout(GDL_DOCK_LAYOUT(layout_),
+                                             0))
+    {
+        g_warning("gdl_dock_layout_load_layout: Failed");
+    }
+}
+
 void
 MainWindow::on_about_activate_link_url(
                 Gtk::AboutDialog & about_dialog,
@@ -548,7 +592,7 @@ MainWindow::on_action_help_about() throw()
     about_dialog.set_transient_for(*this);
 
     DataInputStreamPtr fin;
-    FilePtr file = Gio::File::create_for_path(artistsFile);
+    FilePtr file = Gio::File::create_for_path(artistsFile_);
 
     try
     {
@@ -582,7 +626,7 @@ MainWindow::on_action_help_about() throw()
         about_dialog.set_artists(artists);
     }
 
-    file = Gio::File::create_for_path(authorsFile);
+    file = Gio::File::create_for_path(authorsFile_);
 
     try
     {
@@ -619,7 +663,7 @@ MainWindow::on_action_help_about() throw()
     about_dialog.set_copyright(
         "Copyright \xc2\xa9 2009 - 2010 The " PACKAGE_NAME " authors");
 
-    file = Gio::File::create_for_path(copyingFile);
+    file = Gio::File::create_for_path(copyingFile_);
 
     try
     {
@@ -643,34 +687,16 @@ MainWindow::on_action_help_about() throw()
         about_dialog.set_license(license);
     }
 
-    PixbufPtr logo;
-
-    try
-    {
-        logo = Gdk::Pixbuf::create_from_file(logoFile);
-    }
-    catch (const Glib::FileError & e)
-    {
-        g_warning("%s", e.what().c_str());
-        return;
-    }
-    catch (const Gdk::PixbufError & e)
-    {
-        g_warning("%s", e.what().c_str());
-        return;
-    }
-
-    if (0 != logo)
-    {
-        about_dialog.set_logo(logo);
-    }
+    about_dialog.set_logo_icon_name(PACKAGE_TARNAME);
 
+    // Translators: Put your names here.
     about_dialog.set_translator_credits(_("translator-credits"));
+
     about_dialog.set_version(_(PACKAGE_VERSION));
-    about_dialog.set_website(websiteURL);
-    about_dialog.set_website_label(Glib::ustring::compose(
-                                       _("%1 Website"),
-                                       PACKAGE_NAME));
+    about_dialog.set_website(websiteURL_);
+    about_dialog.set_website_label(
+        Glib::ustring::compose(C_("Project website", "%1 Website"),
+                               PACKAGE_NAME));
     about_dialog.set_wrap_license(false);
     about_dialog.set_url_hook(
         sigc::mem_fun(*this,
@@ -679,6 +705,14 @@ MainWindow::on_action_help_about() throw()
     about_dialog.run();
 }
 
+void
+MainWindow::on_action_help_contents() throw()
+{
+    const ScreenPtr screen = get_screen();
+    gtk_show_uri(screen->gobj(), "ghelp:" PACKAGE_TARNAME,
+                 GDK_CURRENT_TIME, 0);
+}
+
 void
 MainWindow::on_action_photo_quit() throw()
 {
@@ -781,14 +815,14 @@ MainWindow::on_action_view_full_screen(
         {
             g_warning("%s: File not found", layout_file.c_str());
 
-            if (false == Glib::file_test(layoutFile,
+            if (false == Glib::file_test(layoutFile_,
                                          Glib::FILE_TEST_EXISTS))
             {
-                g_warning("%s: File not found", layoutFile.c_str());
+                g_warning("%s: File not found", layoutFile_.c_str());
                 return;
             }
 
-            layout_file = layoutFile;
+            layout_file = layoutFile_;
         }
 
         const bool result = gdl_dock_layout_load_from_file(
@@ -853,6 +887,19 @@ MainWindow::on_delete_event(GdkEventAny * event)
     return return_value;
 }
 
+void
+MainWindow::save_layout() throw()
+{
+    gdl_dock_layout_save_layout(GDL_DOCK_LAYOUT(layout_), 0);
+
+    if (false == gdl_dock_layout_save_to_file(
+                     GDL_DOCK_LAYOUT(layout_),
+                     get_user_layout_file().c_str()))
+    {
+        g_warning("gdl_dock_layout_save_to_file: Failed");
+    }
+}
+
 void
 MainWindow::set_busy(bool busy) throw()
 {
@@ -875,43 +922,4 @@ MainWindow::set_busy(bool busy) throw()
     spinnerToolItem_.set_spinning(busy);
 }
 
-void
-MainWindow::connect_progress( const ProgressObserverPtr &observer ) throw()
-{
-    observer_ = observer;
-    progress_.set_fraction( 0.0L );
-
-    observer_->progress().connect(
-        sigc::mem_fun(*this,
-                      &MainWindow::on_progress));
-
-    observer_->dispatcher_reset().connect(
-        sigc::mem_fun(*this,
-                      &MainWindow::on_reset));
-}
-
-void
-MainWindow::on_progress() throw()
-{
-    const guint64 num_events = observer_->get_num_events();
-    if (0 == num_events)
-    {
-        return;
-    }
-
-    const double percentage =
-        static_cast<double>(observer_->get_current_events())
-        / static_cast<double>(num_events);
-
-    progress_.set_fraction( percentage );
-    progress_.show();
-}
-
-void
-MainWindow::on_reset() throw()
-{
-    progress_.hide();
-    progress_.set_fraction(0.0);
-}
-
 } // namespace Solang
