@@ -27,7 +27,7 @@ NonCopyable::NonCopyable()
 {
 }
 
-NonCopyable::~NonCopyable() throw()
+NonCopyable::~NonCopyable()
 {
 } 
 
