@@ -30,7 +30,8 @@ class Operation :
     public:
         virtual BufferPtr
         apply(const BufferPtr & buffer,
-              const ProgressObserverPtr & observer) throw();
+              const ProgressObserverPtr & observer)
+              throw(Glib::Thread::Exit);
 
     protected:
         virtual
