@@ -33,7 +33,6 @@
 #include "i-renderer-selector.h"
 #include "main-window.h"
 #include "photo.h"
-#include "photo-search-criteria.h"
 #include "pixbuf-maker.h"
 #include "types.h"
 
@@ -41,7 +40,7 @@ namespace Solang
 {
 
 static const std::string uiFile
-    = PACKAGE_DATA_DIR"/"PACKAGE_TARNAME"/ui/"
+    = PACKAGE_DATA_DIR G_DIR_SEPARATOR_S "ui" G_DIR_SEPARATOR_S
           PACKAGE_TARNAME"-enlarged-renderer.ui";
 
 static void
@@ -63,14 +62,14 @@ image_view_on_scroll_event(GtkImageView * view,
 
 EnlargedRenderer::EnlargedRenderer() throw() :
     IRenderer(),
-    sigc::trackable(),
+    Plugin(),
     application_(NULL),
     firstUse_(true),
     iconFactory_(Gtk::IconFactory::create()),
     actionGroup_(0),
     uiID_(0),
     dockItemName_("enlarged-dock-item"),
-    dockItemTitle_(_("Enlarged")),
+    dockItemTitle_(C_("Mode or view", "Enlarged")),
     dockItemBehaviour_(GDL_DOCK_ITEM_BEH_NO_GRIP),
     dockItem_(NULL),
     imageView_(0),
@@ -83,21 +82,21 @@ EnlargedRenderer::EnlargedRenderer() throw() :
     Gtk::IconSource icon_source;
     Gtk::IconSet icon_set_mode_image_edit;
 
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-        "/pixmaps/mode-image-edit-16.png");
+    icon_source.set_filename(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                             "pixmaps" G_DIR_SEPARATOR_S
+                             "mode-image-edit-16.png");
     icon_source.set_size(Gtk::IconSize(16));
     icon_set_mode_image_edit.add_source(icon_source);
 
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-        "/pixmaps/mode-image-edit-22.png");
+    icon_source.set_filename(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                             "pixmaps" G_DIR_SEPARATOR_S
+                             "mode-image-edit-22.png");
     icon_source.set_size(Gtk::IconSize(22));
     icon_set_mode_image_edit.add_source(icon_source);
 
-    icon_source.set_filename(
-        PACKAGE_DATA_DIR"/"PACKAGE_TARNAME
-        "/pixmaps/mode-image-edit-24.png");
+    icon_source.set_filename(PACKAGE_DATA_DIR G_DIR_SEPARATOR_S
+                             "pixmaps" G_DIR_SEPARATOR_S
+                             "mode-image-edit-24.png");
     icon_source.set_size(Gtk::IconSize(24));
     icon_set_mode_image_edit.add_source(icon_source);
 
@@ -115,9 +114,68 @@ void
 EnlargedRenderer::init(Application & application) throw()
 {
     application_ = &application;
+
+    RendererRegistry & renderer_registry
+        = application.get_renderer_registry();
+    renderer_registry.add(this);
+
     // initialized_.emit(*this);
 }
 
+void
+EnlargedRenderer::final(Application & application) throw()
+{
+    if (true == firstUse_)
+    {
+        return;
+    }
+
+    MainWindow & main_window = application.get_main_window();
+    const UIManagerPtr & ui_manager = main_window.get_ui_manager();
+
+    if (0 != uiID_)
+    {
+        ui_manager->remove_action_group(actionGroup_);
+        ui_manager->remove_ui(uiID_);
+        uiID_ = 0;
+    }
+
+    actionGroup_.reset();
+
+    signalMainWindowStateEvent_.disconnect();
+    signalListStoreChangeEnd_.disconnect();
+    signalSwitchPage_.disconnect();
+
+    main_window.undock_object_center(GDL_DOCK_OBJECT(dockItem_));
+
+    pageNum_ = -1;
+    firstUse_ = true;
+
+    RendererRegistry & renderer_registry
+        = application.get_renderer_registry();
+    renderer_registry.remove(this);
+
+    // finalized_.emit(*this);
+}
+
+void
+EnlargedRenderer::visit_renderer(BrowserRenderer & browser_renderer)
+                                 throw()
+{
+}
+
+void
+EnlargedRenderer::visit_renderer(EnlargedRenderer & enlarged_renderer)
+                                 throw()
+{
+}
+
+void
+EnlargedRenderer::visit_renderer(SlideshowRenderer & slideshow_renderer)
+                                 throw()
+{
+}
+
 void
 EnlargedRenderer::render(const PhotoPtr & photo) throw()
 {
@@ -231,38 +289,6 @@ EnlargedRenderer::render(const PhotoList & photos) throw()
 {
 }
 
-void
-EnlargedRenderer::final(Application & application) throw()
-{
-    if (true == firstUse_)
-    {
-        return;
-    }
-
-    MainWindow & main_window = application.get_main_window();
-    const UIManagerPtr & ui_manager = main_window.get_ui_manager();
-
-    if (0 != uiID_)
-    {
-        ui_manager->remove_action_group(actionGroup_);
-        ui_manager->remove_ui(uiID_);
-        uiID_ = 0;
-    }
-
-    actionGroup_.reset();
-
-    signalMainWindowStateEvent_.disconnect();
-    signalListStoreChangeEnd_.disconnect();
-    signalSwitchPage_.disconnect();
-
-    main_window.undock_object_center(GDL_DOCK_OBJECT(dockItem_));
-
-    pageNum_ = -1;
-    firstUse_ = true;
-
-    // finalized_.emit(*this);
-}
-
 void
 EnlargedRenderer::create_action_group() throw()
 {
@@ -272,9 +298,9 @@ EnlargedRenderer::create_action_group() throw()
                                               __LINE__));
 
     actionGroup_->add(
-        Gtk::Action::create(
+        Gtk::Action::create_with_icon_name(
             "ActionViewEnlargedSlideshow",
-            Gtk::StockID(PACKAGE_TARNAME"-slideshow-play"),
+            "slideshow-play",
             _("_Slideshow"),
             _("Start a slideshow view of the photos")),
         Gtk::AccelKey("F5"),
@@ -291,7 +317,7 @@ EnlargedRenderer::create_action_group() throw()
             _("_Zoom In"),
             _("Enlarge the photo"));
 
-        action->property_short_label().set_value(_("In"));
+        action->property_short_label().set_value(C_("Zoom", "In"));
         action->property_is_important().set_value(true);
         actionGroup_->add(
             action, Gtk::AccelKey("<control>plus"),
@@ -305,7 +331,7 @@ EnlargedRenderer::create_action_group() throw()
             _("Zoom _Out"),
             _("Shrink the photo"));
 
-        action->property_short_label().set_value(_("Out"));
+        action->property_short_label().set_value(C_("Zoom", "Out"));
         action->property_is_important().set_value(true);
         actionGroup_->add(
             action, Gtk::AccelKey("<control>minus"),
@@ -319,7 +345,8 @@ EnlargedRenderer::create_action_group() throw()
             _("_Normal Size"),
             _("Show the photo at its normal size"));
 
-        action->property_short_label().set_value(_("Normal"));
+        action->property_short_label().set_value(
+                                           C_("Zoom", "Normal"));
         action->property_is_important().set_value(true);
         actionGroup_->add(
             action, Gtk::AccelKey("<control>0"),
@@ -333,7 +360,7 @@ EnlargedRenderer::create_action_group() throw()
             _("Best _Fit"),
             _("Fit the photo to the window"));
 
-        action->property_short_label().set_value(_("Fit"));
+        action->property_short_label().set_value(C_("Zoom", "Fit"));
         action->property_is_important().set_value(true);
         actionGroup_->add(
             action,
@@ -347,7 +374,8 @@ EnlargedRenderer::create_action_group() throw()
             _("_Previous Photo"),
             _("Go to the previous photo in the collection"));
 
-        action->property_short_label().set_value(_("Previous"));
+        action->property_short_label().set_value(
+            C_("Navigation", "Previous"));
         action->property_is_important().set_value(true);
         actionGroup_->add(
             action, Gtk::AccelKey("<alt>Left"),
@@ -361,7 +389,8 @@ EnlargedRenderer::create_action_group() throw()
             _("_Next Photo"),
             _("Go to the next photo in the collection"));
 
-        action->property_short_label().set_value(_("Next"));
+        action->property_short_label().set_value(
+            C_("Navigation", "Next"));
         action->property_is_important().set_value(true);
         actionGroup_->add(
             action, Gtk::AccelKey("<alt>Right"),
@@ -462,6 +491,13 @@ EnlargedRenderer::create_action_group() throw()
         Gtk::AccelKey("KP_Subtract"),
         sigc::mem_fun(*this,
                       &EnlargedRenderer::on_action_view_zoom_out));
+
+    actionGroup_->add(
+        Gtk::Action::create(
+            "AccelQuitEnlarged", Gtk::StockID()),
+        Gtk::AccelKey("Escape"),
+        sigc::mem_fun(*this,
+                      &EnlargedRenderer::on_action_quit_enlarged));
 }
 
 PhotoList
@@ -480,6 +516,19 @@ EnlargedRenderer::get_current_selection() throw()
     return photos;
 }
 
+TreePathList
+EnlargedRenderer::get_selected_paths() const throw()
+{
+    const TreeModelPtr & tree_model = application_->get_list_store();
+    const Gtk::TreeModel::iterator & iter
+        = application_->get_list_store_iter();
+
+    TreePathList paths;
+    paths.push_back(tree_model->get_path(iter));
+
+    return paths;
+}
+
 void
 EnlargedRenderer::present() throw()
 {
@@ -640,6 +689,23 @@ EnlargedRenderer::on_action_go_last() throw()
     engine.selection_changed().emit();
 }
 
+void
+EnlargedRenderer::on_action_quit_enlarged() throw()
+{
+    RendererRegistry & renderer_registry
+        = application_->get_renderer_registry();
+    const IRendererPtr browser_renderer
+        = renderer_registry.select<BrowserRenderer>();
+
+    if (0 == browser_renderer)
+    {
+        return;
+    }
+
+    browser_renderer->present();
+    // NB: BrowserRenderer::render is a no-op.
+}
+
 void
 EnlargedRenderer::on_action_view_slideshow() throw()
 {
